import webbrowser
from typing import Callable
from .....com.orbitgis.lib.lang.Boolean import Boolean
from .....com.orbitgis.tool.message.Message import Message
from .AMap import AMap
from .Constants import Constants
from .SDKViewer3DM import SDKViewer3DM
from .SDKWebSocketServer import SDKWebSocketServer
from .Viewer3DM import Viewer3DM

# Class SDKViewerHost hosts the 3D Mapping Viewer SDK
#
class SDKViewerHost():

    # The host name of the websocket used to communicate between the QGIS python websocket server and the browser javascript viewer websocket client
    WEBSOCKET_HOST: str = "localhost"
    # The port number of the websocket used to communicate between the QGIS python websocket server and the browser javascript viewer websocket client
    WEBSOCKET_PORT: int = 7201

    # Create a new host for the 3D Mapping Viewer
    #
    def __init__(self):
        self.webSocketServer = None
        self.viewer = None

    # Initialize the viewer (call this method first to create a viewer instance and add signal listeners)
    #
    def initializeViewer(self,pluginName: str,options: AMap[object],lostConnectionHandler: Callable[[],None]) -> None:
        Message.print(__name__,"Creating websocket server")
        self.webSocketServer = SDKWebSocketServer(lostConnectionHandler)
        Message.print(__name__,"Creating sdk viewer, version "+Constants.SDK_VERSION+", date "+Constants.SDK_DATE)
        options.set("useShadowDom",Boolean.FALSE)
        self.viewer = SDKViewer3DM(self.webSocketServer,pluginName,options)
        self.webSocketServer.setCommandHandler(self.viewer.processWebSocketCommand)
        Message.print(__name__,"Starting websocket server")
        self.webSocketServer.startServerThread(SDKViewerHost.WEBSOCKET_HOST,SDKViewerHost.WEBSOCKET_PORT)
        Message.print(__name__,"Started websocket server")

    # Start the viewer (call this method after initializing the viewer)
    #
    def startViewer(self) -> None:
        self.viewer.clearReady()
        Message.print(__name__,"Starting javascript viewer")
        viewerURL = self.viewer.options.get("viewerURL")
        if (viewerURL==None):
            viewerURL = self.viewer.options.get("publisherURL")
        if (viewerURL==None):
            autoUpdate = self.viewer.options.get("autoUpdate")
            if (autoUpdate==None):
                autoUpdate = "true"
            viewerURL = "https://cdn.3dmapping.cloud/"+Constants.SDK_VERSION+"/viewer_sdk/?mode=websocket&protocol=ws&hostname="+SDKViewerHost.WEBSOCKET_HOST+"&port="+str(SDKViewerHost.WEBSOCKET_PORT)+"&endpoint=/endpoint"
            if (autoUpdate!="false"):
                viewerURL = (viewerURL+"&autoupdate=true")
        webbrowser.open(viewerURL)
        Message.print(__name__,"Started javascript viewer '"+viewerURL+"'")
    
    # Check if a websocket connection to the viewer exists
    #
    def hasViewerConnection(self) -> bool:
        return self.webSocketServer.isConnected()
        
    # Get the viewer
    #
    def getViewer(self) -> Viewer3DM:
        return self.viewer.getBridgeViewer()

    # Check if the viewer if ready
    #
    def isViewerReady(self) -> bool:
        return self.viewer.isReady()
